@@ -1,7 +1,5 @@
-#!/usr/bin/env python
 # coding=utf-8
-"""A wrapper for Paramiko that attempts to make ssh sessions easier to work with.  It also contains the
-shell_command function for running local shell scripts!
+"""A wrapper for Paramiko that attempts to make ssh sessions easier to work with.
 """
 # Copyright (C) 2015-2022 Jesse Almanrode
 #
@@ -17,65 +15,71 @@ shell_command function for running local shell scripts!
 #
 #     You should have received a copy of the GNU Lesser General Public License
 #     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-from collections import namedtuple, OrderedDict
-from getpass import getuser
 import logging
 import os
 import paramiko
 import socket
-import sys
+from getpass import getuser
+from typing import Any, Optional
 
-# Privates
-__author__ = 'Jesse Almanrode (jesse@almanrode.com)'
-
-if sys.version_info[0] < 3:
-    raise Exception('Python2.x is no longer supported in this version of sshreader')
-elif sys.version_info[1] < 6:
-    raise Exception('Only Python 3.6 and later is supported in this version of sshreader')
-
-# Globals
-Command = namedtuple('Command', ['cmd', 'stdout', 'stderr', 'return_code'])
+from .types import Command, EnvVars, Timeout
+log = logging.getLogger('sshreader')
 
 
-def envvars():
+def envvars() -> EnvVars:
     """ Attempt to determine the current username and location of any ssh private keys.
     If any value is unable to be determined it is returned as 'None'.
 
     This method also checks for any private keys loaded into the SSH Agent.
 
     :return: NamedTuple of (username, agent_keys, rsa_key, dsa_key, ecdsa_key)
+    :rtype: :class:`typing.NamedTuple`
     """
-    env = OrderedDict(username=None, agent_keys=None, rsa_key=None, dsa_key=None, ecdsa_key=None)
-    EnvVars = namedtuple('EnvVars', env.keys())
-    if os.getlogin() == getuser():
+    global log
+    env = {'username': None, 'rsa_key': None, 'dsa_key': None, 'ecdsa_key': None, 'agent_keys': None}
+    user_home = os.getenv('HOME', '~')
+    if user_home == '~':
+        user_home = os.path.expanduser('~')
+    try:
         env['username'] = getuser()
-    userhome = os.path.expanduser('~')
-    if os.path.exists(userhome + "/.ssh"):
-        keyfiles = os.listdir(userhome + "/.ssh")
+    except OSError:  # Running in a container or inside an IDE, let's take our best guess based on your $HOME
+        env['username'] = user_home.split('/').pop()
+    if os.path.exists(user_home + "/.ssh"):
+        keyfiles = os.listdir(user_home + "/.ssh")
         if "id_rsa" in keyfiles:
-            env['rsa_key'] = userhome + "/.ssh/id_rsa"
+            env['rsa_key'] = user_home + "/.ssh/id_rsa"
         if "id_dsa" in keyfiles:
-            env['dsa_key'] = userhome + "/.ssh/id_dsa"
+            env['dsa_key'] = user_home + "/.ssh/id_dsa"
         if 'id_ecdsa' in keyfiles:
-            env['ecdsa_key'] = userhome + '/.ssh/id_ecdsa'
+            env['ecdsa_key'] = user_home + '/.ssh/id_ecdsa'
     env['agent_keys'] = paramiko.Agent().get_keys()
     return EnvVars(**env)
 
 
 class SSH(object):
-    """SSH Session object
+    """SSH Session class which can be used to send commands to a remote server.  It also supports basic SFTP commands
+    and can be used to push/pull files from a remote system.
 
     :param fqdn: Fully qualified domain name or IP address
+    :type fqdn: str, required
     :param username: SSH username
+    :type username: str, required
     :param password: SSH password
-    :param keyfile: SSH private key file
+    :type password: str, optional
+    :param keyfile: Path to SSH Private Key File
+    :type keyfile: str, optional
     :param keypass: SSH private key password
-    :param port: SSH port
-    :param connect: Initiate the connect
-    :return: SSH connection object
-    :raises: SSHException
+    :type keypass: str, optional
+    :param port: SSH port (Default: 22)
+    :type port: int, optional
+    :param connect: Initiate ssh connection on object initialization (Default: True)
+    :type connect: bool, optional
+    :param rsa_sha2: Enable/Disable RSA w/SHA2 hashes (Default: True)
+    :type rsa_sha2: bool, optional
+    :raises: :class:`paramiko.SSHException`
     """
-    def __init__(self, fqdn, username=None, password=None, keyfile=None, keypass=None, port=22, connect=True):
+    def __init__(self, fqdn: str, username: str, password: Optional[str] = None, keyfile: Optional[str] = None,
+                 keypass: Optional[str] = None, port: int = 22, connect: bool = True, rsa_sha2: bool = True) -> None:
         if not keyfile:
             if len(paramiko.Agent().get_keys()) == 0:
                 if not all((username, password)):
@@ -89,7 +93,7 @@ class SSH(object):
             self.keyfile = os.path.abspath(os.path.expanduser(keyfile))
         else:
             self.keyfile = keyfile
-        self.rsa_sha2 = False  # Enable RSA w/SHA2 hashes (OpenSSH 8.8 and later)
+        self.rsa_sha2 = rsa_sha2
         self.keypass = keypass
         self.port = port
         self._connection = paramiko.SSHClient()
@@ -97,7 +101,7 @@ class SSH(object):
         if connect:
             self.__connect()
 
-    def __str__(self):
+    def __str__(self) -> str:
         return str(self.__dict__)
 
     def __enter__(self):
@@ -108,28 +112,32 @@ class SSH(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.__close()
 
-    def sftp_put(self, srcfile, dstfile):
+    def sftp_put(self, srcfile: str, dstfile: str) -> Any:
         """ Use the SFTP subsystem of OpenSSH to copy a local file to a remote host
 
         :param srcfile: Path to the local file
+        :type srcfile: str, required
         :param dstfile: Path to the remote file
-        :return: Result of paramiko.SFTPClient.put()
+        :type dstfile: str, required
+        :return: Result of :meth:`paramiko.SFTPClient.put()`
         """
         if not self.__alive():
             raise paramiko.SSHException("connection to %s not established" % (self.host,))
         sftp = paramiko.SFTPClient.from_transport(self._connection.get_transport())
         try:
-            result = sftp.put(os.path.expanduser(srcfile), os.path.expanduser(dstfile), confirm=True)
+            result = sftp.put(os.path.expanduser(srcfile), os.path.expanduser(dstfile))
         finally:
             sftp.close()
         return result
 
-    def sftp_get(self, srcfile, dstfile):
+    def sftp_get(self, srcfile: str, dstfile: str) -> None:
         """ Use the SFTP subsystem of OpenSSH to copy a remote file to the localhost
 
         :param srcfile: Path to the remote file
+        :type srcfile: str, required
         :param dstfile: Path to the local file
-        :return: Result of paramiko.SFTPClient.get()
+        :type dstfile: str, required
+        :return: None
         """
         if not self.__alive():
             raise paramiko.SSHException("connection to %s not established" % (self.host,))
@@ -139,15 +147,21 @@ class SSH(object):
         finally:
             sftp.close()
 
-    def ssh_command(self, command, timeout=30, combine=False, decodebytes=True):
+    def ssh_command(self, command: str, timeout: Timeout = 30, combine: bool = False,
+                    decodebytes: bool = True) -> Command:
         """Run a command over an ssh connection
 
         :param command: The command to run
-        :param timeout: Timeout for the command
-        :param combine: Combine stderr and stdout (pseudo TTY)
-        :param decodebytes: Decode bytes objects to unicode strings
+        :type command: str, required
+        :param timeout: Timeout for blocking commands in seconds (Default: 30)
+        :type timeout: int or float, optional
+        :param combine: Combine stderr and stdout using a pseudo TTY (Default: False)
+        :type combine: bool, optional
+        :param decodebytes: Decode bytes objects to unicode strings in Python3 (Default: True)
+        :type decodebytes: bool, optional
         :return: Namedtuple of (cmd, stdout, stderr, return_code) or (cmd, stdout, return_code)
-        :raises: SSHException
+        :rtype: Command
+        :raises: :class:`paramiko.SSHException`
         """
         if self.__alive() is False:
             raise paramiko.SSHException("connection to %s not established" % (self.host, ))
@@ -186,7 +200,8 @@ class SSH(object):
         """Is an SSH connection alive
 
         :return: True or False
-        :raises: SSHException
+        :rtype: bool
+        :raises: :class:`paramiko.SSHException`
         """
         if self._connection.get_transport() is None:
             return False
@@ -201,26 +216,17 @@ class SSH(object):
         """
         return self.__connect()
 
-    def connect(self, failfast=True, timeout=30):
+    def connect(self, timeout: Timeout = 0.5) -> bool:
         """Opens an SSH Connection
 
-        :param failfast: Test socket connection before attempting to connect
-        :param timeout: SSH connection timeout in seconds
+        :param timeout: TCP Timeout in seconds (Default: 0.5)
+        :type timeout: int or float, optional
         :return: True
-        :raises: SSHException
+        :rtype: bool
+        :raises: :class:`paramiko.SSHException`
         """
         if self.__alive():
-            raise paramiko.SSHException("connection to % already established" % (self.host, ))
-        # Fail Fast for unreachable host:port
-        if failfast:
-            try:
-                s = socket.socket()
-                s.settimeout(1)
-                s.connect((self.host, self.port))
-            except socket.timeout:
-                raise paramiko.SSHException('connect to host %s port %d: Operation timed out' % (self.host, self.port))
-            else:
-                s.close()
+            raise paramiko.SSHException("connection to %s already established" % (self.host, ))
         paramiko.util.logging.getLogger().setLevel(logging.CRITICAL)  # Keeping paramiko from logging errors to stdout
         if not self.keyfile:
             if len(paramiko.Agent().get_keys()) == 0:
@@ -229,19 +235,19 @@ class SSH(object):
         if self.keyfile:
             if self.rsa_sha2:
                 self._connection.connect(self.host, port=self.port, username=self.username, password=self.keypass,
-                                        key_filename=self.keyfile, timeout=timeout, look_for_keys=False)
+                                         key_filename=self.keyfile, timeout=timeout, look_for_keys=False)
             else:
                 # While this is more insecure, it is required for pre-OpenSSH 8.8 servers
                 # For more info, visit: https://www.paramiko.org/changelog.html#2.9.0
                 self._connection.connect(self.host, port=self.port, username=self.username, password=self.keypass,
-                                        key_filename=self.keyfile, timeout=timeout, look_for_keys=False, 
-                                        disabled_algorithms={'pubkeys': ['rsa-sha2-256', 'rsa-sha2-512']})
+                                         key_filename=self.keyfile, timeout=timeout, look_for_keys=False,
+                                         disabled_algorithms={'pubkeys': ['rsa-sha2-256', 'rsa-sha2-512']})
         else:
             self._connection.connect(self.host, port=self.port, username=self.username, password=self.password,
                                      timeout=timeout, look_for_keys=False)
         return True
 
-    # Privatizing some of the functions so SSH can be subclassed
+    # Privatizing some functions so SSH can be subclassed
     __alive = alive
     __connect = connect
     __close = close
